using System;
using System.Windows.Forms;
using Plux;
using Plux.Framework;


[Extension(OnCreated = "OnCreated")]
[Plug("Workbench.Dialog", OnPlugged = "Dialog_Plugged")]
[ParamValue("Tag", "CUSTOMIZETOOLBAR")]
public partial class Customize : Form, IControl {
  private ExtensionInfo ext;

  public Customize() {
    InitializeComponent();
  }

  public Control Control { get { return control; } }

  public void OnCreated(object s, ExtensionEventArgs args) {
    control.Close.DialogResult = DialogResult.OK;
    control.Close.Click += Close_Click;
    ext = args.ExtensionInfo;
  }

  public void Dialog_Plugged(object s, PlugEventArgs args) {
    PlugInfo toolbarPlug = PlugInfoEx.FindPlug(args.PlugInfo, "Plux.Toolbar");
    SlotInfo actionSlot = toolbarPlug.ExtensionInfo.FindProvidedSlotTransitive("Plux.Action");
    control.Listbox.Format += Listbox_Format;
    control.Listbox.Attach(actionSlot);
  }

  private static void Listbox_Format(object sender, ListControlConvertEventArgs e) {
    var plugType = (PlugTypeInfo) e.ListItem;
    string s = plugType.ExtensionTypeInfo.Name;
    string desc = (string) plugType.GetParamValue("Description");
    if (!String.IsNullOrEmpty(desc)) {
      s += String.Format(" - {0}", desc);
    }
    e.Value = s;
  }

  private void Close_Click(object sender, EventArgs e) {
    ext.Release();
  }
}


[Extension]
[Plug("Plux.Action")]
[ParamValue("MenuItem", "&View|0.4~&Customize Toolbar...|0.41")]
[ParamValue("LazyLoad", false)]
public class CustomizeToolbar : IAction {
  #region IAction Members

  public void Do(object sender, ActionEventArgs args) {
    SlotInfo toolSlot =
        args.SlotInfo.ExtensionInfo.FindProvidedSlotTransitive("Workbench.Dialog");
    PlugTypeInfo plugType = SlotInfoEx.FindRegisteredPlug(toolSlot, "Tag", "CUSTOMIZETOOLBAR");
    plugType.CreateExtension(toolSlot).Plug(toolSlot);
  }

  public bool IsEnabled(object sender, ActionEventArgs args) {
    SlotInfo toolbarSlot =
        args.SlotInfo.ExtensionInfo.FindProvidedSlotTransitive("Plux.ToolStrip");
    return toolbarSlot.PluggedPlugInfos.Count > 0;
  }

  #endregion
}