using System;
using System.Collections.Generic;
using System.Text;
using Metrix.Contracts;
using Plux;

[Extension("MetrixCommands", OnCreated = "OnCreated")]
[Plug("Commands")]
[Slot("Plux.Figure", Multiple = true, LazyLoad = true)]
public class MetrixCommands : IConsoleCommands {
  MetrixCommand cmd = null;
  #region IConsoleCommands Members
  public IEnumerable<ICommand> Commands {
    get { yield return cmd; }
  }
  #endregion
  public void OnCreated(object s, ExtensionEventArgs args) {
    cmd = new MetrixCommand(args.ExtensionInfo.SlotInfos["Plux.Figure"]);
  }
}

public class MetrixCommand : ICommand {
  readonly SlotInfo slot;
  public MetrixCommand(SlotInfo si) {
    slot = si;
  }
  #region ICommand Members
  public string Name {
    get { return "get-metrix"; }
  }
  public CommandHandler Handler {
    get { return OnCommand; }
  }
  public IEnumerable<string> GetArguments(string[] command, int pos) {
    switch(pos) {
      case 0:
        List<string> figures = new List<string>();
        foreach(PlugTypeInfo plugType in slot.RegisteredPlugTypeInfos)
          figures.Add((string) plugType.GetParamValue("ConsoleCommand"));
        return figures.ToArray();
      default:
        return null;
    }
  }
  public string Help {
    get { return "Displays runtime resource metrics"; }
  }
  #endregion
  public string OnCommand(string[] command) {
    if(!slot.IsOpen || slot.RegisteredPlugTypeInfos.Count == 0)
      return "No figures available";
    if(command.Length < 2) return String.Empty;
    foreach(PlugTypeInfo plugType in slot.RegisteredPlugTypeInfos) {
      if ((string)plugType.GetParamValue("ConsoleCommand")
         == command[1]) {
        return ListFigure(plugType);
      }
    }
    return String.Empty;
  }
  static string ListFigure(PlugTypeInfo plugType) {
    ExtensionInfo ext = plugType.ExtensionTypeInfo.GetSharedExtension(true);
    IFigure f = (IFigure) ext.Object;
    string desc = (string) plugType.GetParamValue("Description");
    int cnt = 0;
    StringBuilder sb = new StringBuilder();
    foreach(IMeasurement m in f.Measurements) {
      sb.AppendFormat("{0}:[{1}] Element={2}, Current={3}, Cumulative={4}{5}",
                      ++cnt, desc, m.Item, m.Current, m.Total,
                      Environment.NewLine);
    }
    return sb.ToString();
  }
}