using System;
using System.Diagnostics;
using System.Linq;

namespace Plux.Framework {
  public static class ExitAction {
    public static void Do(PlugInfo startup) {
      if (startup.ExtensionInfo.IsReleased) {
        return;
      }
      startup.ExtensionInfo.Release();
      if (ReadyForShutdown()) {
        Runtime.Shutdown();
      }
    }

    private static bool ReadyForShutdown() {
      SlotInfo startupSlot = Runtime.Repository.SlotInfos["Startup"];
      foreach (PlugInfo plug in startupSlot.PluggedPlugInfos) {
        PlugTypeInfo plugType = plug.ExtensionInfo.ExtensionTypeInfo.PlugTypeInfos[startupSlot.Name];
        if (!(bool) plugType.GetParamValue("UnplugOnExit")) {
          return false;
        }
      }
      return true;
    }

    public static void Do(object startup) {
      Do(Runtime.GetPlugInfo(startup, "Startup"));
    }

    public static void Do(object sender, EventArgs args) {
      ExtensionInfo ext = Runtime.GetExtensionInfo(sender);
      Debug.Assert(ext != null);
      Do(ext.PlugInfos["Startup"]);
    }

    public static void Do(object s, ActionEventArgs args) {
      ExtensionInfo host = args.SlotInfo.ExtensionInfo;
      SlotInfo startup = Runtime.Repository.SlotInfos["Startup"];
      foreach (Path path in host.FindHostPaths(startup)) {
        Do(path.Connections.Last().Plug);
      }
    }
  }
}