using System;
using System.Collections.Generic;

namespace Plux.Framework {
  [AttributeUsage(AttributeTargets.Class, AllowMultiple = true, Inherited = true
    )]
  public class QueuePlugsAttribute : FrameworkSlotAttribute {
    public int Length;
    public QueuePlugsAttribute(string slot) : base(slot) {}
  }

  public class QueuePlugs {
    readonly QueuePlugsAttribute attr;
    readonly SlotInfo slot;
    public QueuePlugs(SlotInfo slot, QueuePlugsAttribute attr) {
      this.slot = slot;
      this.attr = attr;
      slot.Plugging += OnPlugging;
    }
    void OnPlugging(object sender, CancelPlugEventArgs args) {
        if (!Runtime.IsCompositionAllowed) return;
      if(slot.PluggedPlugInfos.Count == attr.Length)
        slot.Unplug(slot.PluggedPlugInfos[0]);
    }
  }

  class QueuePlugsStartup : SlotStartup<QueuePlugsAttribute> {
    static readonly Dictionary<SlotInfo, QueuePlugs> data =
      new Dictionary<SlotInfo, QueuePlugs>();
    public override void Register(ExtensionTypeInfo extType) {}
    public override void Unregister(ExtensionTypeInfo extType) {}
    public override void SlotCreated(SlotInfo slot, QueuePlugsAttribute attr) {
      data[slot] = new QueuePlugs(slot, attr);
    }
    public override void SlotReleased(SlotInfo slot, QueuePlugsAttribute attr) {
      data.Remove(slot);
    }
  }
}