﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Plux.Framework {
  public static class SlotOpenHistory {
    private static List<SlotInfo> history = new List<SlotInfo>();
    internal static void Init() {
      Runtime.Repository.PluginAdded += PluginAdded;
      Runtime.Repository.PluginRemoved += PluginRemoved;
      foreach(PluginInfo pi in Runtime.Repository.PluginInfos) Register(pi);
    }
    private static void PluginAdded(object sender, PluginEventArgs args) {
      Register(args.PluginInfo);
    }
    private static void PluginRemoved(object sender, PluginEventArgs args) {
      Unregister(args.PluginInfo);
    }
    private static void Register(PluginInfo plugin) {
      foreach(ExtensionTypeInfo type in plugin.ExtensionTypeInfos) {
        foreach(SlotTypeInfo slot in type.SlotTypeInfos) {
          slot.Opened += RecordOpened;
          slot.Closed += RecordClosed;
        }
      }
    }
    private static void Unregister(PluginInfo plugin) {
      foreach(ExtensionTypeInfo type in plugin.ExtensionTypeInfos) {
        foreach(SlotTypeInfo slot in type.SlotTypeInfos) {
          slot.Opened -= RecordOpened;
          slot.Closed -= RecordClosed;
        }
      }
    }
    private static void RecordOpened(object sender, SlotEventArgs args) {
      history.Add(args.SlotInfo);
    }
    private static void RecordClosed(object sender, SlotEventArgs args) {
      history.Remove(args.SlotInfo);
    }
    public static IList<SlotInfo> Slots {
      get {
        return history.AsReadOnly();
      }
    }
    public static List<SlotInfo> GetOpenHistory(ExtensionInfo extension) {
      List<SlotInfo> result = new List<SlotInfo>();
      foreach (SlotInfo slot in history) {
        if (extension == slot.ExtensionInfo) {
          result.Add(slot);
        }
      }
      return result;
    }
  }
}
